/*
 * Copyright 2016 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "globus_gridftp_server_s3_base.h"

globus_i_s3_buffer_t *
globus_i_s3_buffer_get(
    globus_i_s3_transfer_t             *transfer_info,
    size_t                              size)
{
    GlobusS3Enter();

    if (! transfer_info->current_buffer)
    {
        GlobusS3Trace("no current buffer\n");
        if (! transfer_info->free_buffers)
        {
            globus_i_s3_buffer_t       *new_buffer;
            globus_size_t               optimal_blocksize;

            GlobusS3Trace("no free buffers\n");

            globus_gridftp_server_get_block_size(
                    transfer_info->op,
                    &optimal_blocksize);

            if (size < optimal_blocksize)
            {
                GlobusS3Trace("requested_size=%zd optimal_size=%zd\n",
                        size, optimal_blocksize);
                size = optimal_blocksize;
            }

            if (size > (SIZE_MAX - sizeof(globus_i_s3_buffer_t)))
            {
                size = SIZE_MAX - sizeof(globus_i_s3_buffer_t);
            }
            new_buffer = malloc(sizeof(globus_i_s3_buffer_t) + size);
            GlobusS3Debug("new_buffer=%p size=%zd buffer=%p\n",
                    (void *) new_buffer, size, (void *) new_buffer->buffer);

            if (new_buffer == NULL)
            {
                GlobusS3Warning("error allocating buffer\n");
                return NULL;
            }
            new_buffer->buffer_len = size;
            new_buffer->buffer_used = 0;
            new_buffer->next = NULL;
            new_buffer->transfer_offset = UINT64_C(-1);

            transfer_info->free_buffers = new_buffer;
        }

        transfer_info->current_buffer = transfer_info->free_buffers;
        transfer_info->free_buffers = transfer_info->free_buffers->next;
        transfer_info->current_buffer->next = NULL;
    }
    GlobusS3Exit();
    return transfer_info->current_buffer;
}
/* globus_i_s3_buffer_get() */
// vim: ft=c:
